#version 440 core

layout (location = 0) out vec4 FragColor;

in vec2 uv;

uniform sampler2D image;

uniform bool extract;
uniform bool horizontal;
uniform float weight[5] = float[] (0.227027, 0.1945946, 0.1216216, 0.054054, 0.016216);

void main()
{            
    vec4 inColor = texture(image, uv);
    // check whether fragment output is higher than threshold, if so output as brightness color
	if(extract){ //either extract from scene HDR buffer
		float brightness = dot(inColor.rgb, vec3(0.2126, 0.7152, 0.0722));
		if(brightness > 1.0) //could make this branchless by multiplying inColor with bool cast to int, and empty value with negated bool as int
			FragColor = inColor;
		else
			FragColor = vec4(0.0, 0.0, 0.0, 0.0);//using alpha of 0 for propper alpha blending later
	}
	else { //or do blurring
		vec2 tex_offset = 1.0 / textureSize(image, 0); // gets size of single texel
		vec3 result = texture(image, uv).rgb * weight[0]; // current fragment's contribution
		if(horizontal)
		{
			for(int i = 1; i < 5; ++i)
			{
				result += texture(image, uv + vec2(tex_offset.x * i, 0.0)).rgb * weight[i];
				result += texture(image, uv - vec2(tex_offset.x * i, 0.0)).rgb * weight[i];
			}
		}
		else
		{
			for(int i = 1; i < 5; ++i)
			{
				result += texture(image, uv + vec2(0.0, tex_offset.y * i)).rgb * weight[i];
				result += texture(image, uv - vec2(0.0, tex_offset.y * i)).rgb * weight[i];
			}
		}
		FragColor = vec4(result, 1.0);
	}
}