#version 440 core

in VertexData {
	vec3 position_world;
	vec3 normal_world;
} vert;

out vec4 color;

uniform vec3 camera_world;
uniform vec3 materialCoefficients; // x=ambient, y=diffuse, z=specular 
uniform float specularAlpha;
uniform vec3 diffuseColor;
uniform sampler2D textureSampler;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} directionalLight;

//n is the normal vector
//l is the light direction
//v is the view vector (points from surface to eye)
//diffuseCol is the diffuse color
//diffuseCoeff is the material coeffiecent for diffuse reflection
//specularCol is the specular color
//specularCoeff is the material coeffiecent for specular reflection
vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseCol, float diffuseCoeff, vec3 specularCol, float specularCoeff, float alpha, bool attenuate, vec3 attenuation) {
	//length of the light vector
	float d = length(l);
	//normalized light vector
	l = normalize(l);
	//attenuation needs to be defined as one in case we do not attenuate
	float att = 1.0;	
	if(attenuate) //attenuation is only relevant for point light
	{
		att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	}
	vec3 r = reflect(-l, n); // reflect the light vector across the surface normal
	vec3 diffuseComponent = diffuseCoeff * diffuseCol ; //diffuse contribution
	vec3 specularComponent = specularCoeff * specularCol * pow(max(0, dot(r, v)), alpha); //specular contribution
	return (diffuseComponent + specularComponent)*att* max(0, dot(n, l)); //add and attenuate :)
}

void main(){
    //TODO: implement materials system
    vec3 tempMaterials = vec3(0.2,0.85,0.5); // x = ambient, y = diffuse, z = specular
    vec3 tempDirectionalLight = normalize(vec3(-1.0,-0.5,-0.5));
    vec3 tempDirectionalLightColor = vec3(1.f,1.f,1.f);
	vec4 textureColor = vec4(diffuseColor,1.f);
    // vec3 tempColor = vec3(1.0f, 0.5f, 0.2f);
    float tempSpecularAlpha = 2.0f;

	vec3 v = normalize(camera_world - vert.position_world);

    vec3 lighting = phong(vert.normal_world, -tempDirectionalLight, v, textureColor.xyz,
     tempMaterials.y, tempDirectionalLightColor, tempMaterials.z, tempSpecularAlpha, false, vec3(0));
    // color =  vec4(textureColor.xyz * tempMaterials.x + lighting, 1.0) ;
    color =  vec4(textureColor.xyz , 1.0) ;

    // color = vec4(1.0f, 0.5f, 0.2f, 1.0f);
}